"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SolanaCircleBridge = void 0;
const web3_js_1 = require("@solana/web3.js");
const connect_sdk_1 = require("@wormhole-foundation/connect-sdk");
const anchor_1 = require("@project-serum/anchor");
const spl_token_1 = require("@solana/spl-token");
const connect_sdk_solana_1 = require("@wormhole-foundation/connect-sdk-solana");
const utils_1 = require("./utils");
const instructions_1 = require("./utils/instructions");
class SolanaCircleBridge {
    network;
    chain;
    connection;
    contracts;
    tokenMessenger;
    messageTransmitter;
    constructor(network, chain, connection, contracts) {
        this.network = network;
        this.chain = chain;
        this.connection = connection;
        this.contracts = contracts;
        if (network === 'Devnet')
            throw new Error('CircleBridge not supported on Devnet');
        const msgTransmitterAddress = contracts.cctp?.messageTransmitter;
        if (!msgTransmitterAddress)
            throw new Error(`Circle Messenge Transmitter contract for domain ${chain} not found`);
        this.messageTransmitter = (0, utils_1.createReadOnlyMessageTransmitterProgramInterface)(new web3_js_1.PublicKey(msgTransmitterAddress), this.connection);
        const tokenMessengerAddress = contracts.cctp?.tokenMessenger;
        if (!tokenMessengerAddress)
            throw new Error(`Circle Token Messenger contract for domain ${chain} not found`);
        this.tokenMessenger = (0, utils_1.createReadOnlyTokenMessengerProgramInterface)(new web3_js_1.PublicKey(tokenMessengerAddress), this.connection);
    }
    static async fromRpc(provider, config) {
        const [network, chain] = await connect_sdk_solana_1.SolanaPlatform.chainFromRpc(provider);
        const conf = config[chain];
        if (conf.network !== network)
            throw new Error(`Network mismatch: ${conf.network} != ${network}`);
        return new SolanaCircleBridge(network, chain, provider, conf.contracts);
    }
    async *redeem(sender, message, attestation) {
        const usdc = new web3_js_1.PublicKey(connect_sdk_1.circle.usdcContract.get(this.network, this.chain));
        const senderPk = new connect_sdk_solana_1.SolanaAddress(sender).unwrap();
        const [circleMsg, _] = (0, connect_sdk_1.deserializeCircleMessage)(connect_sdk_1.encoding.hex.decode(message));
        const ix = await (0, instructions_1.createReceiveMessageInstruction)(this.messageTransmitter.programId, this.tokenMessenger.programId, usdc, circleMsg, message, attestation, senderPk);
        const { blockhash } = await this.connection.getLatestBlockhash();
        const transaction = new web3_js_1.Transaction();
        transaction.recentBlockhash = blockhash;
        transaction.feePayer = senderPk;
        transaction.add(ix);
        yield this.createUnsignedTx(transaction, 'CircleBridge.Redeem');
    }
    async *transfer(sender, recipient, amount) {
        const usdc = new web3_js_1.PublicKey(connect_sdk_1.circle.usdcContract.get(this.network, this.chain));
        const senderPk = new connect_sdk_solana_1.SolanaAddress(sender).unwrap();
        const senderATA = (0, spl_token_1.getAssociatedTokenAddressSync)(usdc, senderPk);
        const destinationDomain = connect_sdk_1.circle.circleChainId.get(recipient.chain);
        const destinationAddress = recipient.address.toUniversalAddress();
        const ix = await (0, instructions_1.createDepositForBurnInstruction)(this.messageTransmitter.programId, this.tokenMessenger.programId, usdc, destinationDomain, senderPk, senderATA, destinationAddress, amount);
        const { blockhash } = await this.connection.getLatestBlockhash();
        const transaction = new web3_js_1.Transaction();
        transaction.recentBlockhash = blockhash;
        transaction.feePayer = senderPk;
        transaction.add(ix);
        yield this.createUnsignedTx(transaction, 'CircleBridge.Transfer');
    }
    // Fetch the transaction logs and parse the CircleTransferMessage
    async parseTransactionDetails(txid) {
        const tx = await this.connection.getTransaction(txid);
        if (!tx || !tx.meta)
            throw new Error('Transaction not found');
        // this log contains the cctp message information
        const messageTransmitterParser = new anchor_1.EventParser(this.messageTransmitter.programId, this.messageTransmitter.coder);
        const messageLogs = [
            ...messageTransmitterParser.parseLogs(tx.meta.logMessages || []),
        ];
        const message = new Uint8Array(messageLogs[0].data['message']);
        const [msg, hash] = (0, connect_sdk_1.deserializeCircleMessage)(message);
        const { payload: body } = msg;
        const xferSender = body.messageSender;
        const xferReceiver = body.mintRecipient;
        const sendChain = connect_sdk_1.circle.toCircleChain(msg.sourceDomain);
        const rcvChain = connect_sdk_1.circle.toCircleChain(msg.destinationDomain);
        const token = (0, connect_sdk_1.nativeChainAddress)(sendChain, body.burnToken);
        return {
            from: (0, connect_sdk_1.nativeChainAddress)(sendChain, xferSender),
            to: (0, connect_sdk_1.nativeChainAddress)(rcvChain, xferReceiver),
            token: token,
            amount: body.amount,
            messageId: { message: connect_sdk_1.encoding.hex.encode(message), hash },
        };
    }
    createUnsignedTx(txReq, description, parallelizable = false) {
        return new connect_sdk_solana_1.SolanaUnsignedTransaction(txReq, this.network, this.chain, description, parallelizable);
    }
}
exports.SolanaCircleBridge = SolanaCircleBridge;
//# sourceMappingURL=circleBridge.js.map