"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createReceiveMessageInstruction = void 0;
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
const connect_sdk_1 = require("@wormhole-foundation/connect-sdk");
const accounts_1 = require("../accounts");
const program_1 = require("../program");
async function createReceiveMessageInstruction(messageTransmitterProgramId, tokenMessengerProgramId, usdcAddress, circleMessage, message, attestation, payer) {
    const messageBytes = Buffer.from(connect_sdk_1.encoding.hex.decode(message));
    const attestationBytes = Buffer.from(connect_sdk_1.encoding.hex.decode(attestation));
    const solanaUsdcAddress = new web3_js_1.PublicKey(usdcAddress);
    const sourceUsdcAddress = new web3_js_1.PublicKey(circleMessage.payload.burnToken.toUint8Array());
    const receiver = circleMessage.payload.mintRecipient
        .toNative('Solana')
        .unwrap();
    const payerPubkey = payer ? new web3_js_1.PublicKey(payer) : receiver;
    const userTokenAccount = await (0, spl_token_1.getAssociatedTokenAddress)(solanaUsdcAddress, receiver);
    const srcDomain = connect_sdk_1.circle
        .toCircleChainId(circleMessage.sourceDomain)
        .toString();
    // Find pdas
    const messageTransmitterAccount = (0, accounts_1.findProgramAddress)('message_transmitter', messageTransmitterProgramId);
    const tokenMessenger = (0, accounts_1.findProgramAddress)('token_messenger', tokenMessengerProgramId);
    const tokenMinter = (0, accounts_1.findProgramAddress)('token_minter', tokenMessengerProgramId);
    const localToken = (0, accounts_1.findProgramAddress)('local_token', tokenMessengerProgramId, [solanaUsdcAddress]);
    const remoteTokenMessengerKey = (0, accounts_1.findProgramAddress)('remote_token_messenger', tokenMessengerProgramId, [srcDomain]);
    const tokenPair = (0, accounts_1.findProgramAddress)('token_pair', tokenMessengerProgramId, [
        srcDomain,
        sourceUsdcAddress,
    ]);
    const custodyTokenAccount = (0, accounts_1.findProgramAddress)('custody', tokenMessengerProgramId, [solanaUsdcAddress]);
    const authorityPda = (0, accounts_1.findProgramAddress)('message_transmitter_authority', messageTransmitterProgramId).publicKey;
    // Calculate the nonce PDA.
    const maxNoncesPerAccount = 6400n;
    const firstNonce = ((circleMessage.nonce - BigInt(1)) / maxNoncesPerAccount) *
        maxNoncesPerAccount +
        BigInt(1);
    const usedNonces = (0, accounts_1.findProgramAddress)('used_nonces', messageTransmitterProgramId, [srcDomain, firstNonce.toString()]).publicKey;
    // Build the accountMetas list. These are passed as remainingAccounts for the TokenMessengerMinter CPI
    const accountMetas = [];
    accountMetas.push({
        isSigner: false,
        isWritable: false,
        pubkey: tokenMessenger.publicKey,
    });
    accountMetas.push({
        isSigner: false,
        isWritable: false,
        pubkey: remoteTokenMessengerKey.publicKey,
    });
    accountMetas.push({
        isSigner: false,
        isWritable: true,
        pubkey: tokenMinter.publicKey,
    });
    accountMetas.push({
        isSigner: false,
        isWritable: true,
        pubkey: localToken.publicKey,
    });
    accountMetas.push({
        isSigner: false,
        isWritable: false,
        pubkey: tokenPair.publicKey,
    });
    accountMetas.push({
        isSigner: false,
        isWritable: true,
        pubkey: userTokenAccount,
    });
    accountMetas.push({
        isSigner: false,
        isWritable: true,
        pubkey: custodyTokenAccount.publicKey,
    });
    accountMetas.push({
        isSigner: false,
        isWritable: false,
        pubkey: spl_token_1.TOKEN_PROGRAM_ID,
    });
    const messageTransmitterProgram = (0, program_1.createMessageTransmitterProgramInterface)(messageTransmitterProgramId);
    return (messageTransmitterProgram.methods
        .receiveMessage({
        message: messageBytes,
        attestation: attestationBytes,
    })
        .accounts({
        payer: payerPubkey,
        caller: payerPubkey,
        authorityPda,
        messageTransmitter: messageTransmitterAccount.publicKey,
        usedNonces,
        receiver: tokenMessengerProgramId,
        systemProgram: web3_js_1.SystemProgram.programId,
    })
        // Add remainingAccounts needed for TokenMessengerMinter CPI
        .remainingAccounts(accountMetas)
        .transaction());
}
exports.createReceiveMessageInstruction = createReceiveMessageInstruction;
//# sourceMappingURL=receiveMessage.js.map