import { PublicKey, Transaction } from '@solana/web3.js';
import { circle, deserializeCircleMessage, encoding, nativeChainAddress, } from '@wormhole-foundation/connect-sdk';
import { EventParser } from '@project-serum/anchor';
import { getAssociatedTokenAddressSync } from '@solana/spl-token';
import { SolanaAddress, SolanaPlatform, SolanaUnsignedTransaction, } from '@wormhole-foundation/connect-sdk-solana';
import { createReadOnlyMessageTransmitterProgramInterface, createReadOnlyTokenMessengerProgramInterface, } from './utils';
import { createDepositForBurnInstruction, createReceiveMessageInstruction, } from './utils/instructions';
export class SolanaCircleBridge {
    network;
    chain;
    connection;
    contracts;
    tokenMessenger;
    messageTransmitter;
    constructor(network, chain, connection, contracts) {
        this.network = network;
        this.chain = chain;
        this.connection = connection;
        this.contracts = contracts;
        if (network === 'Devnet')
            throw new Error('CircleBridge not supported on Devnet');
        const msgTransmitterAddress = contracts.cctp?.messageTransmitter;
        if (!msgTransmitterAddress)
            throw new Error(`Circle Messenge Transmitter contract for domain ${chain} not found`);
        this.messageTransmitter = createReadOnlyMessageTransmitterProgramInterface(new PublicKey(msgTransmitterAddress), this.connection);
        const tokenMessengerAddress = contracts.cctp?.tokenMessenger;
        if (!tokenMessengerAddress)
            throw new Error(`Circle Token Messenger contract for domain ${chain} not found`);
        this.tokenMessenger = createReadOnlyTokenMessengerProgramInterface(new PublicKey(tokenMessengerAddress), this.connection);
    }
    static async fromRpc(provider, config) {
        const [network, chain] = await SolanaPlatform.chainFromRpc(provider);
        const conf = config[chain];
        if (conf.network !== network)
            throw new Error(`Network mismatch: ${conf.network} != ${network}`);
        return new SolanaCircleBridge(network, chain, provider, conf.contracts);
    }
    async *redeem(sender, message, attestation) {
        const usdc = new PublicKey(circle.usdcContract.get(this.network, this.chain));
        const senderPk = new SolanaAddress(sender).unwrap();
        const [circleMsg, _] = deserializeCircleMessage(encoding.hex.decode(message));
        const ix = await createReceiveMessageInstruction(this.messageTransmitter.programId, this.tokenMessenger.programId, usdc, circleMsg, message, attestation, senderPk);
        const { blockhash } = await this.connection.getLatestBlockhash();
        const transaction = new Transaction();
        transaction.recentBlockhash = blockhash;
        transaction.feePayer = senderPk;
        transaction.add(ix);
        yield this.createUnsignedTx(transaction, 'CircleBridge.Redeem');
    }
    async *transfer(sender, recipient, amount) {
        const usdc = new PublicKey(circle.usdcContract.get(this.network, this.chain));
        const senderPk = new SolanaAddress(sender).unwrap();
        const senderATA = getAssociatedTokenAddressSync(usdc, senderPk);
        const destinationDomain = circle.circleChainId.get(recipient.chain);
        const destinationAddress = recipient.address.toUniversalAddress();
        const ix = await createDepositForBurnInstruction(this.messageTransmitter.programId, this.tokenMessenger.programId, usdc, destinationDomain, senderPk, senderATA, destinationAddress, amount);
        const { blockhash } = await this.connection.getLatestBlockhash();
        const transaction = new Transaction();
        transaction.recentBlockhash = blockhash;
        transaction.feePayer = senderPk;
        transaction.add(ix);
        yield this.createUnsignedTx(transaction, 'CircleBridge.Transfer');
    }
    // Fetch the transaction logs and parse the CircleTransferMessage
    async parseTransactionDetails(txid) {
        const tx = await this.connection.getTransaction(txid);
        if (!tx || !tx.meta)
            throw new Error('Transaction not found');
        // this log contains the cctp message information
        const messageTransmitterParser = new EventParser(this.messageTransmitter.programId, this.messageTransmitter.coder);
        const messageLogs = [
            ...messageTransmitterParser.parseLogs(tx.meta.logMessages || []),
        ];
        const message = new Uint8Array(messageLogs[0].data['message']);
        const [msg, hash] = deserializeCircleMessage(message);
        const { payload: body } = msg;
        const xferSender = body.messageSender;
        const xferReceiver = body.mintRecipient;
        const sendChain = circle.toCircleChain(msg.sourceDomain);
        const rcvChain = circle.toCircleChain(msg.destinationDomain);
        const token = nativeChainAddress(sendChain, body.burnToken);
        return {
            from: nativeChainAddress(sendChain, xferSender),
            to: nativeChainAddress(rcvChain, xferReceiver),
            token: token,
            amount: body.amount,
            messageId: { message: encoding.hex.encode(message), hash },
        };
    }
    createUnsignedTx(txReq, description, parallelizable = false) {
        return new SolanaUnsignedTransaction(txReq, this.network, this.chain, description, parallelizable);
    }
}
//# sourceMappingURL=circleBridge.js.map